#include <EEPROM.h>
#include <Adafruit_GFX.h>
#include <Adafruit_PCD8544.h>
#include <RTClib.h>

#define ENC_A 2 //А энкодера
#define ENC_B 3 //В энкодера
#define ENC_BUTTON 4 //кнопка энкодера
#define BACKLIGHT 10 //Управление подсветкой
#define BUZZER_OUT 13 //Пин подключен к пищалке
#define THERMISTORPIN A6 // пин термометра
#define TEMPERATURENOMINAL 25 // temp. для номинального сопротивления (практически всегда равна 25 C)
#define BCOEFFICIENT 4000 // бета коэффициент термистора (обычно 3000-4000)
#define NUMSAMPLES 10 // сколько показаний используем для определения среднего значения
#define THERMISTORNOMINAL 10000 // сопротивление при 25 градусах по Цельсию
#define SERIESRESISTOR 10000 // сопротивление второго резистора

Adafruit_PCD8544 display = Adafruit_PCD8544(5, 6, 7, 8, 9); //SCLK, DIN, D/C, CS, RST
RTC_DS1307 RTC;
DateTime now;

const int relay[4] = {14, 15, 16, 17};
volatile long newPosition = 0; //переменная валкодера
bool tamper = false; //контроль подсветки
bool set = false; //Режим установки параметра
unsigned int timeCount = 0; //переменная подсветки
unsigned int dsp = 20; //контраст дисплея
unsigned int choice = 1; //переменная выбора
unsigned int settMode = 0; //Режим настройки
unsigned long corrTime = 0; //время старта в секундах от начала суток
unsigned long previousTime = 0;
int adjSecond = 0; //переменная подстройки времени
int day = 0; //переменная дня подстройки времени
int hoursON[4] = {0, 0, 0, 0};
int minutsON[4] = {0, 0, 0, 0};
int hoursOFF[4] = {0, 0, 0, 0};
int minutsOFF[4] = {0, 0, 0, 0};
int tempNTC = -273; //температура

//чтение EEPROM long
unsigned long EEPROM_ulong_read(int addr)
{
  byte raw[4];
  for (byte i = 0; i < 4; i++) raw[i] = EEPROM.read(addr + i);
  unsigned long &num = (unsigned long&)raw;
  return num;
}

//запись EEPROM long
void EEPROM_ulong_write(int addr, unsigned long num)
{
  byte raw[4];
  (unsigned long&)raw = num;
  for (byte i = 0; i < 4; i++) EEPROM.update(addr + i, raw[i]);
}

void EEPROMread()
{
  for (int i; i < 4; i++)
  {
    hoursON[i] = EEPROM.read(i);
    minutsON[i] = EEPROM.read(i + 4);
    hoursOFF[i] = EEPROM.read(i + 8);
    minutsOFF[i] = EEPROM.read(i + 12);
  }
  adjSecond = EEPROM_ulong_read(16);
  dsp = EEPROM_ulong_read(20);
  corrTime = EEPROM_ulong_read(24);
}

void EEPROMwrite()
{
  for (int i; i < 4; i++)
  {
    EEPROM.update(i, hoursON[i]);
    EEPROM.update(i + 4, minutsON[i]);
    EEPROM.update(i + 8, hoursOFF[i]);
    EEPROM.update(i + 12, minutsOFF[i]);
  }
  EEPROM_ulong_write(16, adjSecond);
  EEPROM_ulong_write(20, dsp);
  EEPROM_ulong_write(24, corrTime);
}

void backLight_on() //включение подсветки
{
  timeCount = 0;
  tamper = true;
}

void backlightTimer() //автовыключение подсветки
{
  if (tamper)
  {
    digitalWrite(BACKLIGHT, HIGH);
    timeCount++;
  }
  if (timeCount >= 300) //подобрать по времени подсветки
  {
    digitalWrite(BACKLIGHT, LOW);
    if (!set)choice = 1;
    tamper = false;
    timeCount = 0;
  }
}

void NTCloop()
{
  uint8_t i;
  float average = 0;
  float steinhart;
  int samples[NUMSAMPLES];
  for (i = 0; i < NUMSAMPLES; i++) samples[i] = analogRead(THERMISTORPIN); // сводим показания в вектор
  for (i = 0; i < NUMSAMPLES; i++) average += samples[i]; // рассчитываем среднее значение
  average /= NUMSAMPLES;
  average = 1023 / average - 1; // конвертируем значение в сопротивление
  average = SERIESRESISTOR / average;
  steinhart = average / THERMISTORNOMINAL; // (R/Ro)
  steinhart = log(steinhart); // ln(R/Ro)
  steinhart /= BCOEFFICIENT; // 1/B * ln(R/Ro)
  steinhart += 1.0 / (TEMPERATURENOMINAL + 273.15); // + (1/To)
  steinhart = 1.0 / steinhart; // инвертируем
  steinhart -= 273.15; // конвертируем в градусы по Цельсию
  tempNTC = steinhart;
}

void LCDloop() //цикл дисплея
{
  unsigned int _choice = choice;
  bool _set = set;
  unsigned int SH = 1;
  if (now.second() & 1) SH = 0;
  display.clearDisplay();

  if (settMode == 0) //основной экран
  {
    //отрисивка времени
    display.setCursor(0, 0);
    display.setTextColor(WHITE, BLACK);
    if (now.day() % 32 < 10) display.print ("0");
    display.print(now.day(), DEC);
    display.print(".");
    if (now.month() % 13 < 10) display.print ("0");
    display.print(now.month(), DEC);
    display.print(".");
    display.print(now.year() - 2000, DEC);
    display.print (" ");
    if (now.hour() % 24 < 10) display.print ("0");
    display.print (now.hour(), DEC);
    display.print (":");
    if (now.minute() % 60 < 10) display.print ("0");
    display.print (now.minute(), DEC);
    display.setTextColor(BLACK , WHITE);

    //отрисивка первой строки
    display.setCursor(3, 12);
    if (_choice == 2)
    {
      display.fillRect(0, 10, 11, 11, BLACK);
      display.setTextColor(WHITE, BLACK);
    }
    else
    {
      display.drawRect(0, 10, 11, 11, BLACK);
      display.setTextColor(BLACK, WHITE);
    }
    display.print ("1");

    display.setCursor(14, 12);
    if (_choice == 3)
    {
      display.fillRect(11, 10, 11, 11, BLACK);
      display.setTextColor(WHITE, BLACK);
    }
    else
    {
      display.drawRect(11, 10, 11, 11, BLACK);
      display.setTextColor(BLACK, WHITE);
    }
    display.print ("2");

    display.setCursor(25, 12);
    if (_choice == 4)
    {
      display.fillRect(22, 10, 11, 11, BLACK);
      display.setTextColor(WHITE, BLACK);
    }
    else
    {
      display.drawRect(22, 10, 11, 11, BLACK);
      display.setTextColor(BLACK, WHITE);
    }
    display.print ("3");

    display.setCursor(36, 12);
    if (_choice == 5)
    {
      display.fillRect(33, 10, 11, 11, BLACK);
      display.setTextColor(WHITE, BLACK);
    }
    else
    {
      display.drawRect(33, 10, 11, 11, BLACK);
      display.setTextColor(BLACK, WHITE);
    }
    display.print ("4");

    //отрисивка второй строки
    display.setTextColor(BLACK, WHITE);
    display.setCursor(3, 25);
    if (digitalRead(relay[0]) == HIGH)display.print ("I");
    else display.print ("O");
    display.setCursor(14, 25);
    if (digitalRead(relay[1]) == HIGH)display.print ("I");
    else display.print ("O");
    display.setCursor(25, 25);
    if (digitalRead(relay[2]) == HIGH)display.print ("I");
    else display.print ("O");
    display.setCursor(36, 25);
    if (digitalRead(relay[3]) == HIGH)display.print ("I");
    else display.print ("O");

    //отрисивка температуры
    display.setCursor(48, 15);
    display.setTextSize(2);
    display.setTextColor(BLACK , WHITE);
    if (tempNTC != -273)
    {
      display.print (tempNTC);
      if (tempNTC >= -9)display.print((char)247);
    }
    else display.print ("ERR");
    display.setTextSize(1);

    //отрисивка кнопок "time/sett"
    display.drawRect(0, 38, 38, 10, BLACK);
    display.drawRect(45, 38, 38, 10, BLACK);

    display.setCursor(1, 39);
    if (_choice == 1) display.setTextColor(WHITE, BLACK);
    else display.setTextColor(BLACK, WHITE);
    display.print (" Auto ");

    display.setCursor(46, 39);
    if (_choice == 0) display.setTextColor(WHITE, BLACK);
    else display.setTextColor(BLACK, WHITE);
    display.print (" Sett ");
  }
  if (settMode == 1) //экран автоматики
  {
    //отрисивка первой строки
    display.setCursor(0, 0);
    display.setTextColor(BLACK , WHITE);
    display.print ("1: ");
    if (_choice == 2)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (hoursON[0] % 24 < 10) display.print ("0");
    display.print (hoursON[0]);
    display.setTextColor(BLACK, WHITE);
    display.print (":");
    if (_choice == 3)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (minutsON[0] % 60 < 10) display.print ("0");
    display.print (minutsON[0]);
    display.setTextColor(BLACK, WHITE);
    display.print ("-");
    if (_choice == 4)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (hoursOFF[0] % 24 < 10) display.print ("0");
    display.print (hoursOFF[0]);
    display.setTextColor(BLACK, WHITE);
    display.print (":");
    if (_choice == 5)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (minutsOFF[0] % 60 < 10) display.print ("0");
    display.print (minutsOFF[0]);
    display.setTextColor(BLACK, WHITE);

    //отрисивка второй строки
    display.setCursor(0, 9);
    display.setTextColor(BLACK , WHITE);
    display.print ("2: ");
    if (_choice == 6)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (hoursON[1] % 24 < 10) display.print ("0");
    display.print (hoursON[1]);
    display.setTextColor(BLACK, WHITE);
    display.print (":");
    if (_choice == 7)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (minutsON[1] % 60 < 10) display.print ("0");
    display.print (minutsON[1]);
    display.setTextColor(BLACK, WHITE);
    display.print ("-");
    if (_choice == 8)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (hoursOFF[1] % 24 < 10) display.print ("0");
    display.print (hoursOFF[1]);
    display.setTextColor(BLACK, WHITE);
    display.print (":");
    if (_choice == 9)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (minutsOFF[1] % 60 < 10) display.print ("0");
    display.print (minutsOFF[1]);
    display.setTextColor(BLACK, WHITE);

    //отрисивка третьей строки
    display.setCursor(0, 18);
    display.setTextColor(BLACK , WHITE);
    display.print ("3:");
    display.print((char)30);
    if (_choice == 10)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (hoursON[2] % 24 < 10) display.print ("0");
    display.print (hoursON[2]);
    display.setTextColor(BLACK, WHITE);
    display.print (":");
    if (_choice == 11)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (minutsON[2] % 60 < 10) display.print ("0");
    display.print (minutsON[2]);
    display.setTextColor(BLACK, WHITE);
    display.print((char)31);
    if (_choice == 12)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (hoursOFF[2] % 24 < 10) display.print ("0");
    display.print (hoursOFF[2]);
    display.setTextColor(BLACK, WHITE);
    display.print (":");
    if (_choice == 13)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (minutsOFF[2] % 60 < 10) display.print ("0");
    display.print (minutsOFF[2]);
    display.setTextColor(BLACK, WHITE);

    //отрисивка четвертой строки
    display.setCursor(0, 27);
    display.setTextColor(BLACK , WHITE);
    display.print ("4: ");
    if (_choice == 14)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (hoursON[3] % 100 < 10) display.print ("0");
    display.print (hoursON[3]);
    display.setTextColor(BLACK, WHITE);
    display.print((char)247);
    display.print ("-");
    if (_choice == 15)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (hoursOFF[3] % 100 < 10) display.print ("0");
    display.print (hoursOFF[3]);
    display.setTextColor(BLACK, WHITE);
    display.print((char)247);

    //отрисовка температуры
    display.setCursor(44, 39);
    display.setTextColor(BLACK, WHITE);
    display.print ("t ");
    if (tempNTC != -273)
    {
      display.print (tempNTC);
      display.print((char)247);
    }
    else display.print ("err");
  }
  if (settMode == 2) //экран установки
  {
    //отрисивка первой строки
    display.setCursor(0, 0);
    display.setTextColor(BLACK , WHITE);
    display.print (" DAY: ");
    if (_choice == 2)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (now.day() % 32 < 10) display.print ("0");
    display.print(now.day(), DEC);
    display.setTextColor(BLACK, WHITE);
    display.print (".");
    if (_choice == 3)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (now.month() % 13 < 10) display.print ("0");
    display.print(now.month(), DEC);
    display.setTextColor(BLACK, WHITE);
    display.print (".");
    if (_choice == 4)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    display.print(now.year() - 2000, DEC);
    display.setTextColor(BLACK, WHITE);

    //отрисивка второй строки
    display.setCursor(0, 9);
    display.setTextColor(BLACK , WHITE);
    display.print ("TIME: ");
    if (_choice == 5)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (now.hour() % 24 < 10) display.print ("0");
    display.print (now.hour(), DEC);
    display.setTextColor(BLACK, WHITE);
    display.print (":");
    if (_choice == 6)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (now.minute() % 60 < 10) display.print ("0");
    display.print (now.minute(), DEC);
    display.setTextColor(BLACK, WHITE);
    display.print (":");
    if (_choice == 7)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (now.second() % 60 < 10) display.print ("0");
    display.print (now.second(), DEC);
    display.setTextColor(BLACK, WHITE);

    //отрисивка третьей строки
    display.setCursor(0, 18);
    display.setTextColor(BLACK , WHITE);
    display.print ("tADJ: ");
    if (_choice == 8)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (adjSecond > 0) display.print ("+");
    display.print (adjSecond);
    display.setTextColor(BLACK, WHITE);
    display.print (" sec");

    //отрисивка четвертой строки
    display.setCursor(0, 27);
    display.setTextColor(BLACK , WHITE);
    display.print ("DSPL: ");
    if (_choice == 9)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    display.print (dsp);
    display.setTextColor(BLACK, WHITE);
    display.print (" %");

    //отрисовка времени синхронизации
    unsigned int cHour = corrTime / 3600;
    unsigned int cMinute = (corrTime - cHour * 3600) / 60;
    display.setCursor(44, 39);
    display.setTextColor(BLACK, WHITE);
    display.print((char)230);
    if (cHour % 24 < 10) display.print ("0");
    display.print (cHour);
    display.print (":");
    if (cMinute % 60 < 10) display.print ("0");
    display.print (cMinute);
  }
  if (settMode != 0)
  {
    //отрисивка кнопки "Back"
    display.drawRect(0, 38, 38, 10, BLACK);
    display.setCursor(1, 39);
    if (_choice == 1) display.setTextColor(WHITE, BLACK);
    else display.setTextColor(BLACK, WHITE);
    display.print (" Back ");
  }
  display.display();
}

void doEncoderA() // функция действия вызываемое прерыванием
{
  if (digitalRead(ENC_A) == digitalRead(ENC_B)) newPosition++;
  else newPosition--;
}

void encoderLoop() //цикл обработки енкодера
{
  static volatile long oldPosition;
  unsigned long curPosition = newPosition;
  if (curPosition != oldPosition)
  {
    backLight_on();
    tone(BUZZER_OUT, 4500, 10);

    if (settMode == 0)
    {
      choice = constrain(choice, 0, 5);
      if (curPosition > oldPosition)
      {
        if (choice < 5)choice++;
        else choice = 0;
      }
      if (curPosition < oldPosition)
      {
        if (choice > 0)choice--;
        else choice = 5;
      }
    }

    if (settMode == 1) //настройки автоматизации
    {
      choice = constrain(choice, 1, 15);
      if (curPosition > oldPosition) // прибавляем переменные
      {
        if (set)
        {
          if (choice == 2)hoursON[0]++;
          if (choice == 3)minutsON[0]++;
          if (choice == 4)hoursOFF[0]++;
          if (choice == 5)minutsOFF[0]++;
          if (choice == 6)hoursON[1]++;
          if (choice == 7)minutsON[1]++;
          if (choice == 8)hoursOFF[1]++;
          if (choice == 9)minutsOFF[1]++;
          if (choice == 10)hoursON[2]++;
          if (choice == 11)minutsON[2]++;
          if (choice == 12)hoursOFF[2]++;
          if (choice == 13)minutsOFF[2]++;
          if (choice == 14)hoursON[3]++;
          if (choice == 15)hoursOFF[3]++;
          for (int i = 0; i < 3; i++)
          {
            if (hoursON[i] > 23) hoursON[i] = 0;
            if (hoursOFF[i] > 23) hoursOFF[i] = 0;
            if (minutsON[i] > 59) minutsON[i] = 0;
            if (minutsOFF[i] > 59) minutsOFF[i] = 0;
          }
          if (hoursON[3] > 99) hoursON[3] = 0;
          if (hoursOFF[3] > 99) hoursOFF[3] = 0;
        }
        else
        {
          if (choice < 15)choice++;
          else choice = 1;
        }
      }
      if (curPosition < oldPosition) //уменьшаем переменные
      {
        if (set)
        {
          if (choice == 2)hoursON[0]--;
          if (choice == 3)minutsON[0]--;
          if (choice == 4)hoursOFF[0]--;
          if (choice == 5)minutsOFF[0]--;
          if (choice == 6)hoursON[1]--;
          if (choice == 7)minutsON[1]--;
          if (choice == 8)hoursOFF[1]--;
          if (choice == 9)minutsOFF[1]--;
          if (choice == 10)hoursON[2]--;
          if (choice == 11)minutsON[2]--;
          if (choice == 12)hoursOFF[2]--;
          if (choice == 13)minutsOFF[2]--;
          if (choice == 14)hoursON[3]--;
          if (choice == 15)hoursOFF[3]--;
          for (int i = 0; i < 3; i++)
          {
            if (hoursON[i] < 0) hoursON[i] = 23;
            if (hoursOFF[i] < 0) hoursOFF[i] = 23;
            if (minutsON[i] < 0) minutsON[i] = 59;
            if (minutsOFF[i] < 0) minutsOFF[i] = 59;
          }
          if (hoursON[3] < 0) hoursON[3] = 99;
          if (hoursOFF[3] < 0) hoursOFF[3] = 99;
        }
        else
        {
          if (choice > 1)choice--;
          else choice = 15;
        }
      }
    }
    if (settMode == 2) //настройки устройства
    {
      dsp = constrain(dsp, 1, 100);
      choice = constrain(choice, 1, 9);
      if (curPosition > oldPosition) // прибавляем переменные
      {
        if (set)
        {
          if (choice == 2)RTC.adjust(now + TimeSpan(1, 0, 0, 0));
          if (choice == 3)RTC.adjust(now + TimeSpan(30, 0, 0, 0));
          if (choice == 4)RTC.adjust(now + TimeSpan(365, 0, 0, 0));
          if (choice == 5)RTC.adjust(now + TimeSpan(0, 1, 0, 0));
          if (choice == 6)RTC.adjust(now + TimeSpan(0, 0, 1, 0));
          if (choice == 7)RTC.adjust(now + TimeSpan(0, 0, 0, 1));
          if (choice == 8)adjSecond++;
          if (choice == 9)
          {
            dsp++;
            display.setContrast(dsp);
          }
        }
        else
        {
          if (choice < 9)choice++;
          else choice = 1;
        }
      }
      if (curPosition < oldPosition) //уменьшаем переменные
      {
        if (set)
        {
          if (choice == 2)RTC.adjust(now - TimeSpan(1, 0, 0, 0));
          if (choice == 3)RTC.adjust(now - TimeSpan(30, 0, 0, 0));
          if (choice == 4)RTC.adjust(now - TimeSpan(365, 0, 0, 0));
          if (choice == 5)RTC.adjust(now - TimeSpan(0, 1, 0, 0));
          if (choice == 6)RTC.adjust(now - TimeSpan(0, 0, 1, 0));
          if (choice == 7)RTC.adjust(now - TimeSpan(0, 0, 0, 1));
          if (choice == 8)adjSecond--;
          if (choice == 9)
          {
            dsp--;
            display.setContrast(dsp);
          }
        }
        else
        {
          if (choice > 1)choice--;
          else choice = 9;
        }
      }
      if (curPosition != oldPosition && set && choice >= 6 && choice <= 7)
        corrTime = now.hour() * 60L * 60L + now.minute() * 60L + now.second(); //запоминаем время в секундах
    }
  }
  oldPosition = curPosition;
}

unsigned int duratation(byte button) //получаем время до отпускания кнопки
{
  unsigned int result = 0;
  while (digitalRead(button) == LOW)
  {
    delay(80);
    if (digitalRead(button) == LOW) result += 100;
    if (result > 1100) return 1100; //таймаут нажатия
  }
  return result;
}

void buttonLoop() //цикл кнопки
{
  if (digitalRead(ENC_BUTTON) == LOW)
  {
    backLight_on();
    unsigned int duration = duratation(ENC_BUTTON);
    if (duration > 1000) //длинное нажатие
    {
      tone(BUZZER_OUT, 4500, 200);
      EEPROMwrite();
      LCDloop();
      while (digitalRead(ENC_BUTTON) == LOW) delay(1); //ждём пока отпустят кнопку
    }
    else if (duration > 10)  //короткое нажатие
    {
      tone(BUZZER_OUT, 4500, 20);
      buttonClick();
      while (digitalRead(ENC_BUTTON) == LOW) delay(1); //ждём пока отпустят кнопку
    }
  }
}

void buttonClick() //обработка режима кнопки и перехода по настройкам
{
  if (settMode == 0)
  {
    if (choice == 0)
    {
      settMode = 2;
      choice = 1;
      return;
    }
    if (choice == 1)
    {
      settMode = 1;
      return;
    }
    else if (choice == 2 || choice == 3 || choice == 4 || choice == 5)
    {
      digitalWrite(relay[choice - 2], !digitalRead(relay[choice - 2]));
      return;
    }
  }
  else
  {
    if (choice == 1)
    {
      settMode = 0;
      return;
    }
    else
    {
      set = !set;
      return;
    }
  }
}

void mainLoop() //основной цикл работы
{
  bool disable[4] = {false, false, false, false};
  unsigned long refTime = now.hour() * 60L + now.minute(); //опорное время в минутах
  unsigned long ON = hoursON[2] * 60L + minutsON[2]; //время включения в минутах
  unsigned long OFF = hoursOFF[2] * 60L + minutsOFF[2]; //время выключения в минутах

  for (int i = 0; i < 2; i++)
  {
    if (hoursON[i] == hoursOFF[i] && minutsON[i] == minutsOFF[i])disable[i] = true;
    if (now.hour() == hoursON[i] && now.minute() == minutsON[i] && now.second() == 0 && !disable[i])digitalWrite(relay[i], HIGH);
    else if (now.hour() == hoursOFF[i] && now.minute() == minutsOFF[i] && now.second() == 0 && !disable[i])digitalWrite(relay[i], LOW);
  }
  if (hoursON[2] == 0 && hoursOFF[2] == 0 && minutsON[2] == 0 && minutsOFF[2] == 0)disable[2] = true;
  if (digitalRead(relay[2]) == LOW && !disable[2])
  {
    if (refTime - previousTime >= OFF)
    {
      previousTime = refTime;
      digitalWrite(relay[2], HIGH);
    }
  }
  if (digitalRead(relay[2]) == HIGH && !disable[2])
  {
    if (refTime - previousTime >= ON)
    {
      previousTime = refTime;
      digitalWrite(relay[2], LOW);
    }
  }
  if (hoursON[3] == hoursOFF[3]) disable[3] = true;
  if (tempNTC == hoursON[3] && !disable[3]) digitalWrite(relay[3], HIGH);
  else if (tempNTC == hoursOFF[3]  && !disable[3]) digitalWrite(relay[3], LOW);
}

void autoCorrection() //коррекция времени раз в сутки
{
  unsigned long ref = now.hour() * 60L * 60L + now.minute() * 60L + now.second(); //опорное время в секундах
  if (ref == corrTime && day != now.day())
  {
    day = now.day();
    RTC.adjust(now + TimeSpan(0, 0, 0, adjSecond));
  }
}

void logo() //логотип + версия
{
  display.clearDisplay();
  display.setCursor(7, 7);
  display.setTextSize(2);
  display.print("TeRRaN");
  display.setCursor(28, 36);
  display.setTextSize(1);
  display.print("v_1.1");
  display.display();
  delay(1500);
}

void initOn() //инициализация (восстановление после отключения)
{
  NTCloop();
  now = RTC.now();
  unsigned long ref = now.hour() * 60L + now.minute(); //опорное время в минутах
  for (int i = 0; i < 2; i++)
  {
    long ON = hoursON[i] * 60L + minutsON[i]; //время включения в минутах
    long OFF = hoursOFF[i] * 60L + minutsOFF[i]; //время выключения в минутах
    if (ON - OFF < 0) if (ON <= ref && OFF > ref)digitalWrite(relay[i], HIGH);
    if (ON - OFF > 0) if (ON <= ref && OFF < ref)digitalWrite(relay[i], HIGH);
  }
  if (hoursON[3] - hoursOFF[3] < 0) if (tempNTC <= hoursON[3] && tempNTC < hoursOFF[3]) digitalWrite(relay[3], HIGH);
  if (hoursON[3] - hoursOFF[3] > 0) if (tempNTC >= hoursON[3] && tempNTC > hoursOFF[3]) digitalWrite(relay[3], HIGH);
}

void defaultSett() //сброс настроек в начальные
{
  EEPROMwrite();
  tone(BUZZER_OUT, 4500, 250);
  delay(500);
  tone(BUZZER_OUT, 4500, 250);
  delay(500);
  tone(BUZZER_OUT, 4500, 250);
}

void setup()
{
  pinMode(BUZZER_OUT, OUTPUT);
  pinMode(BACKLIGHT, OUTPUT);
  pinMode(ENC_A, INPUT_PULLUP);
  pinMode(ENC_B, INPUT_PULLUP);
  pinMode(ENC_BUTTON, INPUT_PULLUP);
  pinMode(THERMISTORPIN, INPUT_PULLUP);
  for (int i = 0; i < 4; i++) pinMode(relay[i], OUTPUT);
  attachInterrupt(0, doEncoderA, RISING);
  if (digitalRead(ENC_BUTTON) == LOW) defaultSett();
  EEPROMread();
  RTC.begin();
  display.begin();
  display.setContrast(dsp);
  logo(); //показываем логотип и ждем секунду
  initOn();
}

void loop()
{
  now = RTC.now(); //основное время
  if (settMode == 0)
  {
    mainLoop();
    if (adjSecond != 0)autoCorrection();
  }
  if (now.second() & 1) NTCloop();
  encoderLoop();
  LCDloop();
  buttonLoop();
  backlightTimer();
}
