#include <EEPROM.h>
#include <Adafruit_GFX.h>
#include <Adafruit_PCD8544.h>
#include <RTClib.h>

#define ENC_A 2 //А энкодера
#define ENC_B 3 //В энкодера
#define ENC_BUTTON 4 //кнопка энкодера
#define BACKLIGHT 10 //Управление подсветкой
#define BUZZER_OUT 13 //Пин подключен к пищалке
#define THERMISTORPIN A6 // пин термометра
#define TEMPERATURENOMINAL 25 // temp. для номинального сопротивления (практически всегда равна 25 C)
#define BCOEFFICIENT 4000 // бета коэффициент термистора (обычно 3000-4000)
#define NUMSAMPLES 10 // сколько показаний используем для определения среднего значения
#define THERMISTORNOMINAL 10000 // сопротивление при 25 градусах по Цельсию
#define SERIESRESISTOR 10000 // сопротивление второго резистора

Adafruit_PCD8544 display = Adafruit_PCD8544(5, 6, 7, 8, 9); //SCLK, DIN, D/C, CS, RST

RTC_DS1307 RTC;
DateTime now;

const int relay[4] = {14, 15, 16, 17};
volatile int newPosition = 0; //переменная валкодера
bool tamper = false; //контроль подсветки
bool set = false; //Режим установки параметра
unsigned int timeCount = 0; //переменная подсветки
unsigned int dsp = 50; //контраст дисплея
unsigned int choice = 1; //переменная выбора
unsigned int settMode = 0; //Режим настройки
unsigned long corrTime = 0; //время старта в секундах от начала суток
unsigned long previousTime = 0;
int adjSecond = 0; //переменная подстройки времени
int day = 0; //переменная дня подстройки времени
int ch[8] = {0, 1, 2, 3, 0, 1, 2, 3};
int hoursON[8] = {0, 0, 0, 0, 0, 0, 0, 0};
int minutsON[8] = {0, 0, 0, 0, 0, 0, 0, 0};
int hoursOFF[8] = {0, 0, 0, 0, 0, 0, 0, 0};
int minutsOFF[8] = {0, 0, 0, 0, 0, 0, 0, 0};
int tempNTC = -273; //температура

//чтение EEPROM long
unsigned long EEPROM_ulong_read(int addr)
{
  byte raw[4];
  for (byte i = 0; i < 4; i++) raw[i] = EEPROM.read(addr + i);
  unsigned long &num = (unsigned long&)raw;
  return num;
}

//запись EEPROM long
void EEPROM_ulong_write(int addr, unsigned long num)
{
  byte raw[4];
  (unsigned long&)raw = num;
  for (byte i = 0; i < 4; i++) EEPROM.update(addr + i, raw[i]);
}

void EEPROMread()
{
  for (int i; i < 8; i++)
  {
    ch[i] = EEPROM.read(i);
    hoursON[i] = EEPROM.read(i + 8);
    minutsON[i] = EEPROM.read(i + 16);
    hoursOFF[i] = EEPROM.read(i + 24);
    minutsOFF[i] = EEPROM.read(i + 32);
  }
  adjSecond = EEPROM_ulong_read(40);
  dsp = EEPROM_ulong_read(44);
  corrTime = EEPROM_ulong_read(48);
}

void EEPROMwrite()
{
  for (int i; i < 8; i++)
  {
    EEPROM.update(i, ch[i]);
    EEPROM.update(i + 8, hoursON[i]);
    EEPROM.update(i + 16, minutsON[i]);
    EEPROM.update(i + 24, hoursOFF[i]);
    EEPROM.update(i + 32, minutsOFF[i]);
  }
  EEPROM_ulong_write(40, adjSecond);
  EEPROM_ulong_write(44, dsp);
  EEPROM_ulong_write(48, corrTime);
}

void backLight_on() //включение подсветки
{
  timeCount = 0;
  tamper = true;
}

void backlightTimer() //автовыключение подсветки
{
  if (tamper)
  {
    digitalWrite(BACKLIGHT, HIGH);
    timeCount++;
  }
  if (timeCount >= 300) //подобрать по времени подсветки
  {
    digitalWrite(BACKLIGHT, LOW);
    if (settMode == 0)choice = 1;
    tamper = false;
    timeCount = 0;
  }
}

void NTCloop()
{
  uint8_t i;
  float average = 0;
  float steinhart;
  int samples[NUMSAMPLES];
  for (i = 0; i < NUMSAMPLES; i++) samples[i] = analogRead(THERMISTORPIN); // сводим показания в вектор
  for (i = 0; i < NUMSAMPLES; i++) average += samples[i]; // рассчитываем среднее значение
  average /= NUMSAMPLES;
  average = 1023 / average - 1; // конвертируем значение в сопротивление
  average = SERIESRESISTOR / average;
  steinhart = average / THERMISTORNOMINAL; // (R/Ro)
  steinhart = log(steinhart); // ln(R/Ro)
  steinhart /= BCOEFFICIENT; // 1/B * ln(R/Ro)
  steinhart += 1.0 / (TEMPERATURENOMINAL + 273.15); // + (1/To)
  steinhart = 1.0 / steinhart; // инвертируем
  steinhart -= 273.15; // конвертируем в градусы по Цельсию
  tempNTC = steinhart;
}

void LCDloop() //цикл дисплея
{
  unsigned int _choice = choice;
  unsigned int SH = 1;
  bool _set = set;
  unsigned int T = tempNTC;
  if (now.second() & 1) SH = 0;
  display.clearDisplay();

  if (settMode == 0) //основной экран
  {
    //отрисивка даты и времени
    display.setCursor(0, 0);
    display.setTextColor(WHITE, BLACK);
    if (now.day() % 32 < 10) display.print("0");
    display.print(now.day(), DEC);
    display.print(".");
    if (now.month() % 13 < 10) display.print("0");
    display.print(now.month(), DEC);
    display.print(".");
    display.print(now.year() - 2000, DEC);
    display.print(" ");
    if (now.hour() % 24 < 10) display.print("0");
    display.print(now.hour(), DEC);
    display.print(":");
    if (now.minute() % 60 < 10) display.print("0");
    display.print(now.minute(), DEC);
    display.setTextColor(BLACK , WHITE);

    for (int i = 0; i < 4; i++)
    {
      //отрисивка первой строки
      display.setCursor(i * 11 + 3, 12);
      if (_choice == i + 2)
      {
        display.fillRect(i * 11, 10, 11, 11, BLACK);
        display.setTextColor(WHITE, BLACK);
      }
      else
      {
        display.drawRect(i * 11, 10, 11, 11, BLACK);
        display.setTextColor(BLACK, WHITE);
      }
      display.print(i + 1);
      display.setTextColor(BLACK, WHITE);

      //отрисивка второй строки
      display.setCursor(i * 11 + 3, 25);
      if (digitalRead(relay[i]) == HIGH) display.print("I");
      else display.print("O");
    }

    //отрисивка температуры
    display.setCursor(48, 15);
    display.setTextSize(2);
    display.setTextColor(BLACK , WHITE);
    if (T >= 0)
    {
      if (T < 10) display.print("0");
      display.print(T);
      display.print((char)247);
    }
    else display.print("ERR");
    display.setTextSize(1);

    //отрисивка кнопок "time/sett"
    display.drawRect(0, 38, 38, 10, BLACK);
    display.drawRect(45, 38, 38, 10, BLACK);

    display.setCursor(1, 39);
    if (_choice == 1) display.setTextColor(WHITE, BLACK);
    else display.setTextColor(BLACK, WHITE);
    display.print(" Auto ");

    display.setCursor(46, 39);
    if (_choice == 0) display.setTextColor(WHITE, BLACK);
    else display.setTextColor(BLACK, WHITE);
    display.print(" Sett ");
  }
  if (settMode == 1) //экран автоматики
  {
    int p; //переменная страницы

    if (_choice >= 2 && _choice < 22) p = 0;
    if (_choice >= 22 && _choice < 41) p = 4;
    for (int i = 0; i < 4; i++)
    {
      display.setCursor(0, i * 9); //строка * высоту строки
      display.setTextColor(BLACK, WHITE);
      if (_choice == i * 5 + 2 + p * 5) //строка * количество эл-в управления в строке + положение + переменная страницы * количество эл-в управления в строке
      {
        if (_set) display.setTextColor(SH, !SH);
        else display.setTextColor(WHITE, BLACK);
      }
      display.print("R");
      display.print(ch[i + p] + 1);
      display.setTextColor(BLACK, WHITE);
      display.print(">");
      if (_choice == i * 5 + 3 + p * 5)
      {
        if (_set) display.setTextColor(SH, !SH);
        else display.setTextColor(WHITE, BLACK);
      }
      if (hoursON[i + p] % 24 < 10 && i + p < 6) display.print("0");
      display.print(hoursON[i + p]);
      display.setTextColor(BLACK, WHITE);
      if (i + p >= 6)
      {
        display.print((char)247);
        display.print("-");
      }
      else
      {
        display.print(":");
        if (_choice == i * 5 + 4 + p * 5)
        {
          if (_set) display.setTextColor(SH, !SH);
          else display.setTextColor(WHITE, BLACK);
        }
        if (minutsON[i + p] % 60 < 10) display.print("0");
        display.print(minutsON[i + p]);
        display.setTextColor(BLACK, WHITE);
        display.print("-");
      }
      if (_choice == i * 5 + 5 + p * 5)
      {
        if (_set) display.setTextColor(SH, !SH);
        else display.setTextColor(WHITE, BLACK);
      }
      if (hoursOFF[i + p] % 24 < 10 && i + p < 6) display.print("0");
      display.print (hoursOFF[i + p]);
      display.setTextColor(BLACK, WHITE);
      if (i + p >= 6)
      {
        display.print((char)247);
      }
      else
      {
        display.print(":");
        if (_choice == i * 5 + 6 + p * 5)
        {
          if (_set) display.setTextColor(SH, !SH);
          else display.setTextColor(WHITE, BLACK);
        }
        if (minutsOFF[i + p] % 60 < 10) display.print("0");
        display.print(minutsOFF[i + p]);
      }
    }

    //отрисовка температуры
    display.setCursor(42, 39);
    display.setTextColor(BLACK, WHITE);
    if (T >= 0)
    {
      display.print("t");
      if (T < 10)display.print("0");
      display.print(T);
      display.print((char)247);
    }
    else display.print("err");

    //отрисовка № страницы
    display.setCursor(70, 39);
    display.print("p");
    if (p == 0) display.print("1");
    else display.print("2");
  }
  if (settMode == 2) //экран установки
  {
    //отрисивка первой строки
    display.setCursor(0, 0);
    display.setTextColor(BLACK , WHITE);
    display.print(" DAY: ");
    if (_choice == 2)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (now.day() % 32 < 10) display.print("0");
    display.print(now.day(), DEC);
    display.setTextColor(BLACK, WHITE);
    display.print(".");
    if (_choice == 3)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (now.month() % 13 < 10) display.print("0");
    display.print(now.month(), DEC);
    display.setTextColor(BLACK, WHITE);
    display.print(".");
    if (_choice == 4)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    display.print(now.year() - 2000, DEC);
    display.setTextColor(BLACK, WHITE);

    //отрисивка второй строки
    display.setCursor(0, 9);
    display.setTextColor(BLACK , WHITE);
    display.print("TIME: ");
    if (_choice == 5)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (now.hour() % 24 < 10) display.print("0");
    display.print(now.hour(), DEC);
    display.setTextColor(BLACK, WHITE);
    display.print(":");
    if (_choice == 6)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (now.minute() % 60 < 10) display.print("0");
    display.print(now.minute(), DEC);
    display.setTextColor(BLACK, WHITE);
    display.print(":");
    if (_choice == 7)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (now.second() % 60 < 10) display.print("0");
    display.print(now.second(), DEC);
    display.setTextColor(BLACK, WHITE);

    //отрисивка третьей строки
    display.setCursor(0, 18);
    display.setTextColor(BLACK , WHITE);
    display.print("tADJ: ");
    if (_choice == 8)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    if (adjSecond > 0) display.print("+");
    display.print(adjSecond);
    display.setTextColor(BLACK, WHITE);
    display.print(" sec");

    //отрисивка четвертой строки
    display.setCursor(0, 27);
    display.setTextColor(BLACK , WHITE);
    display.print("DSPL: ");
    if (_choice == 9)
    {
      if (_set) display.setTextColor(SH, !SH);
      else display.setTextColor(WHITE, BLACK);
    }
    display.print(dsp);
    display.setTextColor(BLACK, WHITE);
    display.print(" %");

    //отрисовка времени синхронизации
    unsigned int cHour = corrTime / 3600L;
    unsigned int cMinute = (corrTime - cHour * 3600L) / 60L;
    display.setCursor(44, 39);
    display.setTextColor(BLACK, WHITE);
    display.print((char)230);
    if (cHour % 24 < 10) display.print("0");
    display.print(cHour);
    display.print(":");
    if (cMinute % 60 < 10) display.print("0");
    display.print(cMinute);
  }
  if (settMode != 0)
  {
    //отрисивка кнопки "Back"
    display.drawRect(0, 38, 38, 10, BLACK);
    display.setCursor(1, 39);
    if (_choice == 1) display.setTextColor(WHITE, BLACK);
    else display.setTextColor(BLACK, WHITE);
    display.print(" Back ");
  }
  display.display();
}

void doEncoderA() // функция действия вызываемое прерыванием
{
  if (digitalRead(ENC_A) == digitalRead(ENC_B)) newPosition++;
  else newPosition--;
}

void encoderLoop() //цикл обработки енкодера
{
  static int oldPosition;
  int curPosition = newPosition;
  if (curPosition != oldPosition)
  {
    backLight_on();
    tone(BUZZER_OUT, 4500, 10);

    if (settMode == 0)
    {
      if (curPosition > oldPosition)
      {
        if (choice < 5)choice++;
        else choice = 0;
      }
      if (curPosition < oldPosition)
      {
        if (choice > 0)choice--;
        else choice = 5;
      }
    }

    if (settMode == 1) //настройки автоматизации
    {
      if (curPosition > oldPosition) // прибавляем переменные
      {
        if (set)
        {
          for (int i = 0; i < 8; i++)
          {
            if (choice == i * 5 + 2)ch[i]++;
            if (choice == i * 5 + 3)hoursON[i]++;
            if (choice == i * 5 + 4)minutsON[i]++;
            if (choice == i * 5 + 5)hoursOFF[i]++;
            if (choice == i * 5 + 6)minutsOFF[i]++;
            if (i < 6)
            {
              if (ch[i] > 3) ch[i] = 0;
              if (hoursON[i] > 23) hoursON[i] = 0;
              if (hoursOFF[i] > 23) hoursOFF[i] = 0;
              if (minutsON[i] > 59) minutsON[i] = 0;
              if (minutsOFF[i] > 59) minutsOFF[i] = 0;
            }
            else
            {
              if (hoursON[i] > 99) hoursON[i] = 0;
              if (hoursOFF[i] > 99) hoursOFF[i] = 0;
            }
          }
        }
        else
        {
          if (choice == 33 || choice == 35 || choice == 38)choice++;
          if (choice < 40)choice++;
          else choice = 1;
        }
      }
      if (curPosition < oldPosition) //уменьшаем переменные
      {
        if (set)
        {
          for (int i = 0; i < 8; i++)
          {
            if (choice == i * 5 + 2)ch[i]--;
            if (choice == i * 5 + 3)hoursON[i]--;
            if (choice == i * 5 + 4)minutsON[i]--;
            if (choice == i * 5 + 5)hoursOFF[i]--;
            if (choice == i * 5 + 6)minutsOFF[i]--;
            if (i < 6)
            {
              if (ch[i] < 0)ch[i] = 3;
              if (hoursON[i] < 0) hoursON[i] = 23;
              if (hoursOFF[i] < 0) hoursOFF[i] = 23;
              if (minutsON[i] < 0) minutsON[i] = 59;
              if (minutsOFF[i] < 0) minutsOFF[i] = 59;
            }
            else
            {
              if (hoursON[i] < 0) hoursON[i] = 99;
              if (hoursOFF[i] < 0) hoursOFF[i] = 99;
            }
          }
        }
        else
        {
          if (choice == 35 || choice == 37 || choice == 40)choice--;
          if (choice > 1)choice--;
          else choice = 40;
        }
      }
    }
    if (settMode == 2) //настройки устройства
    {
      dsp = constrain(dsp, 1, 100);
      if (curPosition > oldPosition) // прибавляем переменные
      {
        if (set)
        {
          if (choice == 2)RTC.adjust(now + TimeSpan(1, 0, 0, 0));
          if (choice == 3)RTC.adjust(now + TimeSpan(30, 0, 0, 0));
          if (choice == 4)RTC.adjust(now + TimeSpan(365, 0, 0, 0));
          if (choice == 5)RTC.adjust(now + TimeSpan(0, 1, 0, 0));
          if (choice == 6)RTC.adjust(now + TimeSpan(0, 0, 1, 0));
          if (choice == 7)RTC.adjust(now + TimeSpan(0, 0, 0, 1));
          if (choice == 8)adjSecond++;
          if (choice == 9)
          {
            dsp++;
            display.setContrast(dsp);
          }
        }
        else
        {
          if (choice < 9)choice++;
          else choice = 1;
        }
      }
      if (curPosition < oldPosition) //уменьшаем переменные
      {
        if (set)
        {
          if (choice == 2)RTC.adjust(now - TimeSpan(1, 0, 0, 0));
          if (choice == 3)RTC.adjust(now - TimeSpan(30, 0, 0, 0));
          if (choice == 4)RTC.adjust(now - TimeSpan(365, 0, 0, 0));
          if (choice == 5)RTC.adjust(now - TimeSpan(0, 1, 0, 0));
          if (choice == 6)RTC.adjust(now - TimeSpan(0, 0, 1, 0));
          if (choice == 7)RTC.adjust(now - TimeSpan(0, 0, 0, 1));
          if (choice == 8)adjSecond--;
          if (choice == 9)
          {
            dsp--;
            display.setContrast(dsp);
          }
        }
        else
        {
          if (choice > 1)choice--;
          else choice = 9;
        }
      }
      if (curPosition != oldPosition && set && choice >= 6 && choice <= 7)
        corrTime = now.hour() * 60L * 60L + now.minute() * 60L + now.second(); //запоминаем время в секундах
    }
  }
  oldPosition = curPosition;
}

unsigned int duratation(byte button) //получаем время до отпускания кнопки
{
  unsigned int result = 0;
  while (digitalRead(button) == LOW)
  {
    delay(80);
    if (digitalRead(button) == LOW) result += 100;
    if (result > 1100) return 1100; //таймаут нажатия
  }
  return result;
}

void buttonLoop() //цикл кнопки
{
  if (digitalRead(ENC_BUTTON) == LOW)
  {
    backLight_on();
    unsigned int duration = duratation(ENC_BUTTON);
    if (duration > 1000) //длинное нажатие
    {
      tone(BUZZER_OUT, 4500, 200);
      EEPROMwrite();
      LCDloop();
      while (digitalRead(ENC_BUTTON) == LOW) delay(1); //ждём пока отпустят кнопку
    }
    else if (duration > 10)  //короткое нажатие
    {
      tone(BUZZER_OUT, 4500, 20);
      buttonClick();
      while (digitalRead(ENC_BUTTON) == LOW) delay(1); //ждём пока отпустят кнопку
    }
  }
}

void buttonClick() //обработка режима кнопки и перехода по настройкам
{
  if (settMode == 0)
  {
    if (choice == 0)
    {
      settMode = 2;
      choice = 1;
      return;
    }
    if (choice == 1)
    {
      settMode = 1;
      return;
    }
    else if (choice == 2 || choice == 3 || choice == 4 || choice == 5)
    {
      digitalWrite(relay[choice - 2], !digitalRead(relay[choice - 2]));
      return;
    }
  }
  else
  {
    if (choice == 1)
    {
      settMode = 0;
      return;
    }
    else
    {
      set = !set;
      return;
    }
  }
}

void mainLoop() //основной цикл работы
{
  bool disable[8] = {false, false, false, false, false, false, false, false};
  for (int i = 0; i < 8; i++) //выполнение заданий
  {
    if (i < 6)
    {
      if (hoursON[i] == hoursOFF[i] && minutsON[i] == minutsOFF[i])disable[i] = true;
      if (now.hour() == hoursON[i] && now.minute() == minutsON[i] && now.second() == 0 && !disable[i])digitalWrite(relay[ch[i]], HIGH);
      else if (now.hour() == hoursOFF[i] && now.minute() == minutsOFF[i] && now.second() == 0 && !disable[i])digitalWrite(relay[ch[i]], LOW);
    }
    else
    {
      if (hoursON[i] == hoursOFF[i]) disable[i] = true;
      if (tempNTC == hoursON[i] && !disable[i]) digitalWrite(relay[ch[i]], HIGH);
      else if (tempNTC == hoursOFF[i]  && !disable[i]) digitalWrite(relay[ch[i]], LOW);
    }
  }
}

void autoCorrection() //коррекция времени в часах
{
  unsigned long ref = now.hour() * 60L * 60L + now.minute() * 60L + now.second(); //опорное время в секундах
  if (ref == corrTime && day != now.day())
  {
    day = now.day();
    RTC.adjust(now + TimeSpan(0, 0, 0, adjSecond));
  }
}

void logo() //логотип + версия
{
  display.clearDisplay();
  display.setCursor(7, 7);
  display.setTextSize(2);
  display.print("TeRRaN");
  display.setCursor(28, 36);
  display.setTextSize(1);
  display.print("v_1.2");
  display.display();
  delay(1500);
}

void initOn() //инициализация (восстановление после отключения)
{
  NTCloop();
  now = RTC.now();
  unsigned long ref = now.hour() * 60L + now.minute(); //опорное время в минутах
  for (int i = 0; i < 8; i++)
  {
    if (i < 6)
    {
      long ON = hoursON[i] * 60L + minutsON[i]; //время включения в минутах
      long OFF = hoursOFF[i] * 60L + minutsOFF[i]; //время выключения в минутах
      if (ON - OFF < 0) if (ref >= ON && ref < OFF)digitalWrite(relay[ch[i]], HIGH);
      if (ON - OFF > 0) if ((ref >= ON && ref > OFF) || (ref < OFF && ref < ON))digitalWrite(relay[ch[i]], HIGH);
    }
    else
    {
      if (hoursON[i] - hoursOFF[i] < 0) if (tempNTC <= hoursON[i])digitalWrite(relay[ch[i]], HIGH);
      if (hoursON[i] - hoursOFF[i] > 0) if (tempNTC >= hoursON[i])digitalWrite(relay[ch[i]], HIGH);
    }
  }
}

void defaultSett() //сброс настроек в начальные
{
  for (int i = 0; i < EEPROM.length(); i++) EEPROM.write(i, 0);
  EEPROMwrite();
  tone(BUZZER_OUT, 4500, 250);
  delay(500);
  tone(BUZZER_OUT, 4500, 250);
  delay(500);
  tone(BUZZER_OUT, 4500, 250);
}

void setup()
{
  pinMode(BUZZER_OUT, OUTPUT);
  pinMode(BACKLIGHT, OUTPUT);
  pinMode(ENC_A, INPUT_PULLUP);
  pinMode(ENC_B, INPUT_PULLUP);
  pinMode(ENC_BUTTON, INPUT_PULLUP);
  pinMode(THERMISTORPIN, INPUT_PULLUP);
  for (int i = 0; i < 4; i++) pinMode(relay[i], OUTPUT);
  attachInterrupt(0, doEncoderA, RISING);
  if (digitalRead(ENC_BUTTON) == LOW) defaultSett();
  EEPROMread();
  RTC.begin();
  //RTC.adjust(DateTime(__DATE__, __TIME__));
  display.begin();
  display.setContrast(dsp);
  logo(); //показываем логотип и ждем секунду
  initOn();
}

void loop()
{
  now = RTC.now(); //основное время
  if (settMode == 0)
  {
    mainLoop();
    if (adjSecond != 0)autoCorrection();
  }
  if (now.second() & 1) NTCloop();
  encoderLoop();
  LCDloop();
  buttonLoop();
  backlightTimer();
}
